# Autogenerated by nbdev

d = { 'settings': { 'branch': 'master',
                'doc_baseurl': '/',
                'doc_host': 'https://fastmonai.no',
                'git_url': 'https://github.com/MMIV-ML/fastMONAI',
                'lib_path': 'fastMONAI'},
  'syms': { 'fastMONAI.dataset_info': { 'fastMONAI.dataset_info.MedDataset': ('dataset_info.html#meddataset', 'fastMONAI/dataset_info.py'),
                                        'fastMONAI.dataset_info.MedDataset.__init__': ( 'dataset_info.html#meddataset.__init__',
                                                                                        'fastMONAI/dataset_info.py'),
                                        'fastMONAI.dataset_info.MedDataset._create_data_frame': ( 'dataset_info.html#meddataset._create_data_frame',
                                                                                                  'fastMONAI/dataset_info.py'),
                                        'fastMONAI.dataset_info.MedDataset._get_data_info': ( 'dataset_info.html#meddataset._get_data_info',
                                                                                              'fastMONAI/dataset_info.py'),
                                        'fastMONAI.dataset_info.MedDataset.get_largest_img_size': ( 'dataset_info.html#meddataset.get_largest_img_size',
                                                                                                    'fastMONAI/dataset_info.py'),
                                        'fastMONAI.dataset_info.MedDataset.suggestion': ( 'dataset_info.html#meddataset.suggestion',
                                                                                          'fastMONAI/dataset_info.py'),
                                        'fastMONAI.dataset_info.MedDataset.summary': ( 'dataset_info.html#meddataset.summary',
                                                                                       'fastMONAI/dataset_info.py'),
                                        'fastMONAI.dataset_info.get_class_weights': ( 'dataset_info.html#get_class_weights',
                                                                                      'fastMONAI/dataset_info.py')},
            'fastMONAI.external_data': { 'fastMONAI.external_data.MURLs': ('external_data.html#murls', 'fastMONAI/external_data.py'),
                                         'fastMONAI.external_data._create_nodule_df': ( 'external_data.html#_create_nodule_df',
                                                                                        'fastMONAI/external_data.py'),
                                         'fastMONAI.external_data._create_spine_df': ( 'external_data.html#_create_spine_df',
                                                                                       'fastMONAI/external_data.py'),
                                         'fastMONAI.external_data._dataset_available_locally': ( 'external_data.html#_dataset_available_locally',
                                                                                                 'fastMONAI/external_data.py'),
                                         'fastMONAI.external_data._df_sort_and_add_columns': ( 'external_data.html#_df_sort_and_add_columns',
                                                                                               'fastMONAI/external_data.py'),
                                         'fastMONAI.external_data._process_ixi_xls': ( 'external_data.html#_process_ixi_xls',
                                                                                       'fastMONAI/external_data.py'),
                                         'fastMONAI.external_data._process_medmnist_img': ( 'external_data.html#_process_medmnist_img',
                                                                                            'fastMONAI/external_data.py'),
                                         'fastMONAI.external_data.download_example_endometrial_cancer_data': ( 'external_data.html#download_example_endometrial_cancer_data',
                                                                                                               'fastMONAI/external_data.py'),
                                         'fastMONAI.external_data.download_example_spine_data': ( 'external_data.html#download_example_spine_data',
                                                                                                  'fastMONAI/external_data.py'),
                                         'fastMONAI.external_data.download_ixi_data': ( 'external_data.html#download_ixi_data',
                                                                                        'fastMONAI/external_data.py'),
                                         'fastMONAI.external_data.download_ixi_tiny': ( 'external_data.html#download_ixi_tiny',
                                                                                        'fastMONAI/external_data.py'),
                                         'fastMONAI.external_data.download_medmnist3d_dataset': ( 'external_data.html#download_medmnist3d_dataset',
                                                                                                  'fastMONAI/external_data.py'),
                                         'fastMONAI.external_data.download_spine_test_data': ( 'external_data.html#download_spine_test_data',
                                                                                               'fastMONAI/external_data.py')},
            'fastMONAI.research_utils': { 'fastMONAI.research_utils.pred_postprocess': ( 'research_utils.html#pred_postprocess',
                                                                                         'fastMONAI/research_utils.py')},
            'fastMONAI.utils': { 'fastMONAI.utils.load_variables': ('utils.html#load_variables', 'fastMONAI/utils.py'),
                                 'fastMONAI.utils.print_colab_gpu_info': ('utils.html#print_colab_gpu_info', 'fastMONAI/utils.py'),
                                 'fastMONAI.utils.store_variables': ('utils.html#store_variables', 'fastMONAI/utils.py')},
            'fastMONAI.vision_all': {},
            'fastMONAI.vision_augmentation': { 'fastMONAI.vision_augmentation.BinaryConverter': ( 'vision_augment.html#binaryconverter',
                                                                                                  'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.BinaryConverter.encodes': ( 'vision_augment.html#binaryconverter.encodes',
                                                                                                          'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.BraTSMaskConverter': ( 'vision_augment.html#bratsmaskconverter',
                                                                                                     'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.BraTSMaskConverter.encodes': ( 'vision_augment.html#bratsmaskconverter.encodes',
                                                                                                             'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.CustomDictTransform': ( 'vision_augment.html#customdicttransform',
                                                                                                      'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.CustomDictTransform.__init__': ( 'vision_augment.html#customdicttransform.__init__',
                                                                                                               'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.CustomDictTransform.encodes': ( 'vision_augment.html#customdicttransform.encodes',
                                                                                                              'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.OneOf': ( 'vision_augment.html#oneof',
                                                                                        'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.OneOf.__init__': ( 'vision_augment.html#oneof.__init__',
                                                                                                 'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.PadOrCrop': ( 'vision_augment.html#padorcrop',
                                                                                            'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.PadOrCrop.__init__': ( 'vision_augment.html#padorcrop.__init__',
                                                                                                     'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.PadOrCrop.encodes': ( 'vision_augment.html#padorcrop.encodes',
                                                                                                    'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomAffine': ( 'vision_augment.html#randomaffine',
                                                                                               'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomAffine.__init__': ( 'vision_augment.html#randomaffine.__init__',
                                                                                                        'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomBiasField': ( 'vision_augment.html#randombiasfield',
                                                                                                  'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomBiasField.__init__': ( 'vision_augment.html#randombiasfield.__init__',
                                                                                                           'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomBiasField.encodes': ( 'vision_augment.html#randombiasfield.encodes',
                                                                                                          'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomBlur': ( 'vision_augment.html#randomblur',
                                                                                             'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomBlur.__init__': ( 'vision_augment.html#randomblur.__init__',
                                                                                                      'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomBlur.encodes': ( 'vision_augment.html#randomblur.encodes',
                                                                                                     'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomElasticDeformation': ( 'vision_augment.html#randomelasticdeformation',
                                                                                                           'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomElasticDeformation.__init__': ( 'vision_augment.html#randomelasticdeformation.__init__',
                                                                                                                    'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomFlip': ( 'vision_augment.html#randomflip',
                                                                                             'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomFlip.__init__': ( 'vision_augment.html#randomflip.__init__',
                                                                                                      'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomGamma': ( 'vision_augment.html#randomgamma',
                                                                                              'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomGamma.__init__': ( 'vision_augment.html#randomgamma.__init__',
                                                                                                       'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomGamma.encodes': ( 'vision_augment.html#randomgamma.encodes',
                                                                                                      'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomGhosting': ( 'vision_augment.html#randomghosting',
                                                                                                 'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomGhosting.__init__': ( 'vision_augment.html#randomghosting.__init__',
                                                                                                          'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomGhosting.encodes': ( 'vision_augment.html#randomghosting.encodes',
                                                                                                         'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomMotion': ( 'vision_augment.html#randommotion',
                                                                                               'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomMotion.__init__': ( 'vision_augment.html#randommotion.__init__',
                                                                                                        'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomMotion.encodes': ( 'vision_augment.html#randommotion.encodes',
                                                                                                       'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomNoise': ( 'vision_augment.html#randomnoise',
                                                                                              'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomNoise.__init__': ( 'vision_augment.html#randomnoise.__init__',
                                                                                                       'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomNoise.encodes': ( 'vision_augment.html#randomnoise.encodes',
                                                                                                      'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomSpike': ( 'vision_augment.html#randomspike',
                                                                                              'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomSpike.__init__': ( 'vision_augment.html#randomspike.__init__',
                                                                                                       'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.RandomSpike.encodes': ( 'vision_augment.html#randomspike.encodes',
                                                                                                      'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.ZNormalization': ( 'vision_augment.html#znormalization',
                                                                                                 'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.ZNormalization.__init__': ( 'vision_augment.html#znormalization.__init__',
                                                                                                          'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.ZNormalization.encodes': ( 'vision_augment.html#znormalization.encodes',
                                                                                                         'fastMONAI/vision_augmentation.py'),
                                               'fastMONAI.vision_augmentation.do_pad_or_crop': ( 'vision_augment.html#do_pad_or_crop',
                                                                                                 'fastMONAI/vision_augmentation.py')},
            'fastMONAI.vision_core': { 'fastMONAI.vision_core.MedBase': ('vision_core.html#medbase', 'fastMONAI/vision_core.py'),
                                       'fastMONAI.vision_core.MedBase.__repr__': ( 'vision_core.html#medbase.__repr__',
                                                                                   'fastMONAI/vision_core.py'),
                                       'fastMONAI.vision_core.MedBase.create': ( 'vision_core.html#medbase.create',
                                                                                 'fastMONAI/vision_core.py'),
                                       'fastMONAI.vision_core.MedBase.item_preprocessing': ( 'vision_core.html#medbase.item_preprocessing',
                                                                                             'fastMONAI/vision_core.py'),
                                       'fastMONAI.vision_core.MedBase.show': ('vision_core.html#medbase.show', 'fastMONAI/vision_core.py'),
                                       'fastMONAI.vision_core.MedImage': ('vision_core.html#medimage', 'fastMONAI/vision_core.py'),
                                       'fastMONAI.vision_core.MedMask': ('vision_core.html#medmask', 'fastMONAI/vision_core.py'),
                                       'fastMONAI.vision_core.MetaResolver': ('vision_core.html#metaresolver', 'fastMONAI/vision_core.py'),
                                       'fastMONAI.vision_core._load_and_preprocess': ( 'vision_core.html#_load_and_preprocess',
                                                                                       'fastMONAI/vision_core.py'),
                                       'fastMONAI.vision_core._multi_channel': ( 'vision_core.html#_multi_channel',
                                                                                 'fastMONAI/vision_core.py'),
                                       'fastMONAI.vision_core._preprocess': ('vision_core.html#_preprocess', 'fastMONAI/vision_core.py'),
                                       'fastMONAI.vision_core.med_img_reader': ( 'vision_core.html#med_img_reader',
                                                                                 'fastMONAI/vision_core.py')},
            'fastMONAI.vision_data': { 'fastMONAI.vision_data.MedDataBlock': ('vision_data.html#meddatablock', 'fastMONAI/vision_data.py'),
                                       'fastMONAI.vision_data.MedDataBlock.__init__': ( 'vision_data.html#meddatablock.__init__',
                                                                                        'fastMONAI/vision_data.py'),
                                       'fastMONAI.vision_data.MedImageDataLoaders': ( 'vision_data.html#medimagedataloaders',
                                                                                      'fastMONAI/vision_data.py'),
                                       'fastMONAI.vision_data.MedImageDataLoaders.from_df': ( 'vision_data.html#medimagedataloaders.from_df',
                                                                                              'fastMONAI/vision_data.py'),
                                       'fastMONAI.vision_data.MedMaskBlock': ('vision_data.html#medmaskblock', 'fastMONAI/vision_data.py'),
                                       'fastMONAI.vision_data.batch_pred_to_multiclass_mask': ( 'vision_data.html#batch_pred_to_multiclass_mask',
                                                                                                'fastMONAI/vision_data.py'),
                                       'fastMONAI.vision_data.plot_top_losses': ( 'vision_data.html#plot_top_losses',
                                                                                  'fastMONAI/vision_data.py'),
                                       'fastMONAI.vision_data.pred_to_binary_mask': ( 'vision_data.html#pred_to_binary_mask',
                                                                                      'fastMONAI/vision_data.py'),
                                       'fastMONAI.vision_data.pred_to_multiclass_mask': ( 'vision_data.html#pred_to_multiclass_mask',
                                                                                          'fastMONAI/vision_data.py'),
                                       'fastMONAI.vision_data.show_batch': ('vision_data.html#show_batch', 'fastMONAI/vision_data.py'),
                                       'fastMONAI.vision_data.show_results': ('vision_data.html#show_results', 'fastMONAI/vision_data.py')},
            'fastMONAI.vision_inference': { 'fastMONAI.vision_inference._do_resize': ( 'vision_inference.html#_do_resize',
                                                                                       'fastMONAI/vision_inference.py'),
                                            'fastMONAI.vision_inference._to_original_orientation': ( 'vision_inference.html#_to_original_orientation',
                                                                                                     'fastMONAI/vision_inference.py'),
                                            'fastMONAI.vision_inference._update_uid': ( 'vision_inference.html#_update_uid',
                                                                                        'fastMONAI/vision_inference.py'),
                                            'fastMONAI.vision_inference.compute_binary_tumor_volume': ( 'vision_inference.html#compute_binary_tumor_volume',
                                                                                                        'fastMONAI/vision_inference.py'),
                                            'fastMONAI.vision_inference.gradio_image_classifier': ( 'vision_inference.html#gradio_image_classifier',
                                                                                                    'fastMONAI/vision_inference.py'),
                                            'fastMONAI.vision_inference.inference': ( 'vision_inference.html#inference',
                                                                                      'fastMONAI/vision_inference.py'),
                                            'fastMONAI.vision_inference.load_system_resources': ( 'vision_inference.html#load_system_resources',
                                                                                                  'fastMONAI/vision_inference.py'),
                                            'fastMONAI.vision_inference.refine_binary_pred_mask': ( 'vision_inference.html#refine_binary_pred_mask',
                                                                                                    'fastMONAI/vision_inference.py'),
                                            'fastMONAI.vision_inference.save_series_pred': ( 'vision_inference.html#save_series_pred',
                                                                                             'fastMONAI/vision_inference.py')},
            'fastMONAI.vision_loss': { 'fastMONAI.vision_loss.CustomLoss': ( 'vision_loss_functions.html#customloss',
                                                                             'fastMONAI/vision_loss.py'),
                                       'fastMONAI.vision_loss.CustomLoss.__call__': ( 'vision_loss_functions.html#customloss.__call__',
                                                                                      'fastMONAI/vision_loss.py'),
                                       'fastMONAI.vision_loss.CustomLoss.__init__': ( 'vision_loss_functions.html#customloss.__init__',
                                                                                      'fastMONAI/vision_loss.py'),
                                       'fastMONAI.vision_loss.CustomLoss.activation': ( 'vision_loss_functions.html#customloss.activation',
                                                                                        'fastMONAI/vision_loss.py'),
                                       'fastMONAI.vision_loss.CustomLoss.decodes': ( 'vision_loss_functions.html#customloss.decodes',
                                                                                     'fastMONAI/vision_loss.py'),
                                       'fastMONAI.vision_loss.TverskyFocalLoss': ( 'vision_loss_functions.html#tverskyfocalloss',
                                                                                   'fastMONAI/vision_loss.py'),
                                       'fastMONAI.vision_loss.TverskyFocalLoss.__init__': ( 'vision_loss_functions.html#tverskyfocalloss.__init__',
                                                                                            'fastMONAI/vision_loss.py'),
                                       'fastMONAI.vision_loss.TverskyFocalLoss.forward': ( 'vision_loss_functions.html#tverskyfocalloss.forward',
                                                                                           'fastMONAI/vision_loss.py')},
            'fastMONAI.vision_metrics': { 'fastMONAI.vision_metrics.binary_dice_score': ( 'vision_metrics.html#binary_dice_score',
                                                                                          'fastMONAI/vision_metrics.py'),
                                          'fastMONAI.vision_metrics.binary_hausdorff_distance': ( 'vision_metrics.html#binary_hausdorff_distance',
                                                                                                  'fastMONAI/vision_metrics.py'),
                                          'fastMONAI.vision_metrics.calculate_dsc': ( 'vision_metrics.html#calculate_dsc',
                                                                                      'fastMONAI/vision_metrics.py'),
                                          'fastMONAI.vision_metrics.calculate_haus': ( 'vision_metrics.html#calculate_haus',
                                                                                       'fastMONAI/vision_metrics.py'),
                                          'fastMONAI.vision_metrics.multi_dice_score': ( 'vision_metrics.html#multi_dice_score',
                                                                                         'fastMONAI/vision_metrics.py'),
                                          'fastMONAI.vision_metrics.multi_hausdorff_distance': ( 'vision_metrics.html#multi_hausdorff_distance',
                                                                                                 'fastMONAI/vision_metrics.py')},
            'fastMONAI.vision_plot': { 'fastMONAI.vision_plot._get_slice': ('vision_plot.html#_get_slice', 'fastMONAI/vision_plot.py'),
                                       'fastMONAI.vision_plot.find_max_slice': ( 'vision_plot.html#find_max_slice',
                                                                                 'fastMONAI/vision_plot.py'),
                                       'fastMONAI.vision_plot.show_med_img': ('vision_plot.html#show_med_img', 'fastMONAI/vision_plot.py'),
                                       'fastMONAI.vision_plot.validate_anatomical_plane': ( 'vision_plot.html#validate_anatomical_plane',
                                                                                            'fastMONAI/vision_plot.py')}}}



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/03_vision_augment.ipynb.

# %% auto 0
__all__ = ['CustomDictTransform', 'do_pad_or_crop', 'PadOrCrop', 'ZNormalization', 'BraTSMaskConverter', 'BinaryConverter',
           'RandomGhosting', 'RandomSpike', 'RandomNoise', 'RandomBiasField', 'RandomBlur', 'RandomGamma',
           'RandomMotion', 'RandomElasticDeformation', 'RandomAffine', 'RandomFlip', 'OneOf']

# %% ../nbs/03_vision_augment.ipynb 2
from fastai.data.all import *
from .vision_core import *
import torchio as tio

# %% ../nbs/03_vision_augment.ipynb 5
class CustomDictTransform(ItemTransform):
    """A class that serves as a wrapper to perform an identical transformation on both 
    the image and the target (if it's a mask).
    """
    
    split_idx = 0  # Only perform transformations on training data. Use TTA() for transformations on validation data.

    def __init__(self, aug):
        """Constructs CustomDictTransform object.

        Args:
            aug (Callable): Function to apply augmentation on the image.
        """
        self.aug = aug

    def encodes(self, x):
        """
        Applies the stored transformation to an image, and the same random transformation 
        to the target if it is a mask. If the target is not a mask, it returns the target as is.

        Args:
            x (Tuple[MedImage, Union[MedMask, TensorCategory]]): A tuple containing the 
            image and the target.

        Returns:
            Tuple[MedImage, Union[MedMask, TensorCategory]]: The transformed image and target. 
            If the target is a mask, it's transformed identically to the image. If the target 
            is not a mask, the original target is returned.
        """
        img, y_true = x

        if isinstance(y_true, (MedMask)):
            aug = self.aug(tio.Subject(img=tio.ScalarImage(tensor=img, affine=MedImage.affine_matrix), 
                                        mask=tio.LabelMap(tensor=y_true, affine=MedImage.affine_matrix)))
            return MedImage.create(aug['img'].data), MedMask.create(aug['mask'].data)

        aug = self.aug(tio.Subject(img=tio.ScalarImage(tensor=img)))
        return MedImage.create(aug['img'].data), y_true


# %% ../nbs/03_vision_augment.ipynb 7
def do_pad_or_crop(o, target_shape, padding_mode, mask_name, dtype=torch.Tensor):
    #TODO:refactorize
    pad_or_crop = tio.CropOrPad(target_shape=target_shape, padding_mode=padding_mode, mask_name=mask_name)
    return dtype(pad_or_crop(o))

# %% ../nbs/03_vision_augment.ipynb 8
class PadOrCrop(DisplayedTransform):
    """Resize image using TorchIO `CropOrPad`."""
    
    order = 0

    def __init__(self, size, padding_mode=0, mask_name=None):
        if not is_listy(size): 
            size = [size, size, size]
        self.pad_or_crop = tio.CropOrPad(target_shape=size,
                                    padding_mode=padding_mode, 
                                    mask_name=mask_name)

    def encodes(self, o: (MedImage, MedMask)):
        return type(o)(self.pad_or_crop(o))

# %% ../nbs/03_vision_augment.ipynb 9
class ZNormalization(DisplayedTransform):
    """Apply TorchIO `ZNormalization`."""

    order = 0

    def __init__(self, masking_method=None, channel_wise=True):
        self.z_normalization = tio.ZNormalization(masking_method=masking_method)
        self.channel_wise = channel_wise

    def encodes(self, o: MedImage):
        if self.channel_wise:
            o = torch.stack([self.z_normalization(c[None])[0] for c in o])
        else: o = self.z_normalization(o) 

        return MedImage.create(o)

    def encodes(self, o: MedMask):
        return o

# %% ../nbs/03_vision_augment.ipynb 10
class BraTSMaskConverter(DisplayedTransform):
    '''Convert BraTS masks.'''

    order=1

    def encodes(self, o:(MedImage)): return o

    def encodes(self, o:(MedMask)):
        o = torch.where(o==4, 3., o)
        return MedMask.create(o)

# %% ../nbs/03_vision_augment.ipynb 11
class BinaryConverter(DisplayedTransform):
    '''Convert to binary mask.'''

    order=1

    def encodes(self, o: MedImage): 
        return o

    def encodes(self, o: MedMask):
        o = torch.where(o>0, 1., 0)
        return MedMask.create(o)

# %% ../nbs/03_vision_augment.ipynb 12
class RandomGhosting(DisplayedTransform):
    """Apply TorchIO `RandomGhosting`."""
    
    split_idx, order = 0, 1

    def __init__(self, intensity=(0.5, 1), p=0.5):
        self.add_ghosts = tio.RandomGhosting(intensity=intensity, p=p)

    def encodes(self, o: MedImage):
        return MedImage.create(self.add_ghosts(o))

    def encodes(self, o: MedMask):
        return o

# %% ../nbs/03_vision_augment.ipynb 13
class RandomSpike(DisplayedTransform):
    '''Apply TorchIO `RandomSpike`.'''
    
    split_idx,order=0,1

    def __init__(self, num_spikes=1, intensity=(1, 3), p=0.5):
        self.add_spikes = tio.RandomSpike(num_spikes=num_spikes, intensity=intensity, p=p)

    def encodes(self, o:MedImage): 
        return MedImage.create(self.add_spikes(o))
        
    def encodes(self, o:MedMask):
        return o

# %% ../nbs/03_vision_augment.ipynb 14
class RandomNoise(DisplayedTransform):
    '''Apply TorchIO `RandomNoise`.'''

    split_idx,order=0,1

    def __init__(self, mean=0, std=(0, 0.25), p=0.5):
        self.add_noise = tio.RandomNoise(mean=mean, std=std, p=p)

    def encodes(self, o: MedImage): 
        return MedImage.create(self.add_noise(o))
    
    def encodes(self, o: MedMask):
        return o

# %% ../nbs/03_vision_augment.ipynb 15
class RandomBiasField(DisplayedTransform):
    '''Apply TorchIO `RandomBiasField`.'''

    split_idx,order=0,1

    def __init__(self, coefficients=0.5, order=3, p=0.5):
        self.add_biasfield = tio.RandomBiasField(coefficients=coefficients, order=order, p=p)

    def encodes(self, o: MedImage): 
        return MedImage.create(self.add_biasfield(o))
        
    def encodes(self, o: MedMask):
        return o

# %% ../nbs/03_vision_augment.ipynb 16
class RandomBlur(DisplayedTransform):
    '''Apply TorchIO `RandomBiasField`.'''

    split_idx,order=0,1

    def __init__(self, std=(0, 2), p=0.5):
        self.add_blur = tio.RandomBlur(std=std, p=p)
        
    def encodes(self, o: MedImage): 
        return MedImage.create(self.add_blur(o))
    
    def encodes(self, o: MedMask):
        return o

# %% ../nbs/03_vision_augment.ipynb 17
class RandomGamma(DisplayedTransform):
    '''Apply TorchIO `RandomGamma`.'''


    split_idx,order=0,1

    def __init__(self, log_gamma=(-0.3, 0.3), p=0.5):
        self.add_gamma = tio.RandomGamma(log_gamma=log_gamma, p=p)

    def encodes(self, o: MedImage): 
        return MedImage.create(self.add_gamma(o))
    
    def encodes(self, o: MedMask):
        return o

# %% ../nbs/03_vision_augment.ipynb 18
class RandomMotion(DisplayedTransform):
    """Apply TorchIO `RandomMotion`."""

    split_idx, order = 0, 1

    def __init__(
            self, 
            degrees=10, 
            translation=10, 
            num_transforms=2, 
            image_interpolation='linear', 
            p=0.5
        ):
        self.add_motion = tio.RandomMotion(
            degrees=degrees, 
            translation=translation, 
            num_transforms=num_transforms, 
            image_interpolation=image_interpolation, 
            p=p
        )

    def encodes(self, o: MedImage):
        return MedImage.create(self.add_motion(o))

    def encodes(self, o: MedMask):
        return o

# %% ../nbs/03_vision_augment.ipynb 20
class RandomElasticDeformation(CustomDictTransform):
    """Apply TorchIO `RandomElasticDeformation`."""

    def __init__(self, num_control_points=7, max_displacement=7.5,
                 image_interpolation='linear', p=0.5):
                     
        super().__init__(tio.RandomElasticDeformation(
            num_control_points=num_control_points,
            max_displacement=max_displacement,
            image_interpolation=image_interpolation,
            p=p))

# %% ../nbs/03_vision_augment.ipynb 21
class RandomAffine(CustomDictTransform):
    """Apply TorchIO `RandomAffine`."""

    def __init__(self, scales=0, degrees=10, translation=0, isotropic=False,
                 image_interpolation='linear', default_pad_value=0., p=0.5):
                     
        super().__init__(tio.RandomAffine(
            scales=scales,
            degrees=degrees,
            translation=translation,
            isotropic=isotropic,
            image_interpolation=image_interpolation,
            default_pad_value=default_pad_value,
            p=p))

# %% ../nbs/03_vision_augment.ipynb 22
class RandomFlip(CustomDictTransform):
    """Apply TorchIO `RandomFlip`."""

    def __init__(self, axes='LR', p=0.5):
        super().__init__(tio.RandomFlip(axes=axes, flip_probability=p))

# %% ../nbs/03_vision_augment.ipynb 23
class OneOf(CustomDictTransform):
    """Apply only one of the given transforms using TorchIO `OneOf`."""

    def __init__(self, transform_dict, p=1):
        super().__init__(tio.OneOf(transform_dict, p=p))



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_vision_plot.ipynb.

# %% auto 0
__all__ = ['validate_anatomical_plane', 'show_med_img', 'find_max_slice']

# %% ../nbs/00_vision_plot.ipynb 1
from fastai.data.all import *
from torchio.visualization import rotate

# %% ../nbs/00_vision_plot.ipynb 3
def _get_slice(image, channel: int, slice_index: int, anatomical_plane: int, voxel_size: (int, list)):
    """
    A private method to get a 2D tensor and aspect ratio for plotting.
    This is modified code from the torchio function `plot_volume`.

    Args:
        image: The input image.
        channel: Channel of the image.
        slice_index: Index of the 2D slice.
        anatomical_plane: Anatomical plane of the image.
        voxel_size: Voxel size for the image.

    Returns:
        A 2D tensor containing the sliced image and its aspect.
    """
    if voxel_size is None:
        warnings.warn('Voxel size not defined. Aspect ratio of the plot might not be correct.')
        voxel_size = 1

    if not is_listy(voxel_size):
        voxel_size = (voxel_size, voxel_size, voxel_size)

    sr, sa, ss = voxel_size
    data = image.data[channel]

    if slice_index is None:
        slice_center = np.array(data.shape) // 2
        slice_index = slice_center[anatomical_plane]

    operations = {
        0: (lambda d, i: d[i, :, :], ss / sa),
        1: (lambda d, i: d[:, i, :], ss / sr),
        2: (lambda d, i: d[:, :, i], sa / sr)
    }

    sliced_function, aspect_ratio = operations.get(anatomical_plane)
    sliced_img = rotate(sliced_function(data, slice_index), radiological=True, n=1)

    return sliced_img, aspect_ratio

# %% ../nbs/00_vision_plot.ipynb 4
def validate_anatomical_plane(anatomical_plane):
    """Ensure anatomical_plane is either 0, 1, or 2."""
    if anatomical_plane not in [0, 1, 2]:
        raise ValueError("anatomical_plane should be either 0, 1, or 2.")

# %% ../nbs/00_vision_plot.ipynb 5
@delegates(plt.Axes.imshow, keep=True, but=['shape', 'imlim'])
def show_med_img(im, ctx, channel: int, slice_index: int, anatomical_plane: int,
                 voxel_size: (int, list), ax=None, figsize=None, title=None, **kwargs):
    """Show an image on `ax`. This is a modified code from the fastai function `show_image`.

    Args:
        im: The input image.
        ctx: The context.
        channel: Channel of the image.
        slice_index: Index of the 2D slice.
        anatomical_plane: Anatomical plane of the image.
        voxel_size: Voxel size for the image.
        ax: Axis for the plot.
        figsize: Figure size for the plot.
        title: Title for the plot.
        kwargs: Additional parameters for plt.Axes.imshow method.

    Returns:
        Axis with the plot.
    """
                     
    validate_anatomical_plane(anatomical_plane)
                     
    if hasattrs(im, ('data', 'cpu', 'permute')): # Check if `im` has the necessary attributes
        im = im.data.cpu()
        im, aspect = _get_slice(
            im, 
            channel=channel, 
            anatomical_plane=anatomical_plane,
            voxel_size=voxel_size, 
            slice_index=slice_index
        )

    ax = ax if ax is not None else ctx 

    if ax is None: # ax is only None when .show() is used.
        _, ax = plt.subplots(figsize=figsize)

    ax.imshow(im, aspect=aspect, **kwargs)

    if title is not None:
        ax.set_title(title)

    ax.axis('off')

    return ax

# %% ../nbs/00_vision_plot.ipynb 6
def find_max_slice(mask_data, anatomical_plane):
    """Find slice index based on mask"""
    
    axis_mapping = {0: (1, 2),
                    1: (0, 2),
                    2: (0, 1)}

    sums = mask_data.sum(axis=axis_mapping.get(anatomical_plane, (1, 2)))
    idx = np.argmax(sums)

    return idx



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/05_vision_metrics.ipynb.

# %% auto 0
__all__ = ['calculate_dsc', 'calculate_haus', 'binary_dice_score', 'multi_dice_score', 'binary_hausdorff_distance',
           'multi_hausdorff_distance']

# %% ../nbs/05_vision_metrics.ipynb 1
import torch
import numpy as np
from monai.metrics import compute_hausdorff_distance, compute_dice
from .vision_data import pred_to_binary_mask, batch_pred_to_multiclass_mask

# %% ../nbs/05_vision_metrics.ipynb 3
def calculate_dsc(pred: torch.Tensor, targ: torch.Tensor) -> torch.Tensor:
    """MONAI `compute_meandice`"""

    return torch.Tensor([compute_dice(p[None], t[None]) for p, t in list(zip(pred,targ))])

# %% ../nbs/05_vision_metrics.ipynb 4
def calculate_haus(pred: torch.Tensor, targ: torch.Tensor) -> torch.Tensor:
    """MONAI `compute_hausdorff_distance`"""

    return torch.Tensor([compute_hausdorff_distance(p[None], t[None]) for p, t in list(zip(pred,targ))])

# %% ../nbs/05_vision_metrics.ipynb 5
def binary_dice_score(act: torch.tensor, targ: torch.Tensor) -> torch.Tensor:
    """Calculates the mean Dice score for binary semantic segmentation tasks.
    
    Args:
        act: Activation tensor with dimensions [B, C, W, H, D].
        targ: Target masks with dimensions [B, C, W, H, D].

    Returns:
        Mean Dice score.
    """
    pred = pred_to_binary_mask(act)
    dsc = calculate_dsc(pred.cpu(), targ.cpu())

    return torch.mean(dsc)

# %% ../nbs/05_vision_metrics.ipynb 6
def multi_dice_score(act: torch.Tensor, targ: torch.Tensor) -> torch.Tensor:
    """Calculate the mean Dice score for each class in multi-class semantic 
    segmentation tasks.

    Args:
        act: Activation tensor with dimensions [B, C, W, H, D].
        targ: Target masks with dimensions [B, C, W, H, D].

    Returns:
        Mean Dice score for each class.
    """
    pred, n_classes = batch_pred_to_multiclass_mask(act)
    binary_dice_scores = []

    for c in range(1, n_classes):
        c_pred, c_targ = torch.where(pred == c, 1, 0), torch.where(targ == c, 1, 0)
        dsc = calculate_dsc(c_pred, c_targ)
        binary_dice_scores.append(np.nanmean(dsc)) # #TODO update torch to get torch.nanmean() to work

    return torch.Tensor(binary_dice_scores)

# %% ../nbs/05_vision_metrics.ipynb 7
def binary_hausdorff_distance(act: torch.Tensor, targ: torch.Tensor) -> torch.Tensor:
    """Calculate the mean Hausdorff distance for binary semantic segmentation tasks.
    
    Args:
        act: Activation tensor with dimensions [B, C, W, H, D].
        targ: Target masks with dimensions [B, C, W, H, D].

    Returns:
        Mean Hausdorff distance.
    """
    

    pred = pred_to_binary_mask(act)

    haus = calculate_haus(pred.cpu(), targ.cpu())
    return torch.mean(haus)

# %% ../nbs/05_vision_metrics.ipynb 8
def multi_hausdorff_distance(act: torch.Tensor, targ: torch.Tensor) -> torch.Tensor :
    """Calculate the mean Hausdorff distance for each class in multi-class semantic segmentation tasks.
    
    Args:
        act: Activation tensor with dimensions [B, C, W, H, D].
        targ: Target masks with dimensions [B, C, W, H, D].

    Returns:
        Mean Hausdorff distance for each class.
    """

    pred, n_classes = batch_pred_to_multiclass_mask(act)
    binary_haus = []

    for c in range(1, n_classes):
        c_pred, c_targ = torch.where(pred==c, 1, 0), torch.where(targ==c, 1, 0)
        haus = calculate_haus(pred, targ)
        binary_haus.append(np.nanmean(haus))
    return torch.Tensor(binary_haus)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/07_utils.ipynb.

# %% auto 0
__all__ = ['store_variables', 'load_variables', 'print_colab_gpu_info']

# %% ../nbs/07_utils.ipynb 1
import pickle
import torch
from pathlib import Path

# %% ../nbs/07_utils.ipynb 3
def store_variables(pkl_fn: (str, Path), size: list, reorder: bool, resample: (int, list)):
    """Save variable values in a pickle file."""
    
    var_vals = [size, reorder, resample]
    
    with open(pkl_fn, 'wb') as f:
        pickle.dump(var_vals, f)

# %% ../nbs/07_utils.ipynb 4
def load_variables(pkl_fn: (str, Path)):
    """Loads stored variable values from a pickle file.

    Args:
        pkl_fn: File path of the pickle file to be loaded.

    Returns:
        The deserialized value of the pickled data.
    """
    with open(pkl_fn, 'rb') as f:
        return pickle.load(f)

# %% ../nbs/07_utils.ipynb 5
def print_colab_gpu_info(): 
    """Check if we have a GPU attached to the runtime."""
    
    colab_gpu_msg =(f"{'#'*80}\n"
                    "Remember to attach a GPU to your Colab Runtime:"
                    "\n1. From the **Runtime** menu select **Change Runtime Type**"
                    "\n2. Choose **GPU** from the drop-down menu"
                    "\n3. Click **'SAVE'**\n"
                    f"{'#'*80}")
    
    if torch.cuda.is_available(): print('GPU attached.')
    else: print(colab_gpu_msg)



# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/08_dataset_info.ipynb.

# %% auto 0
__all__ = ['MedDataset', 'get_class_weights']

# %% ../nbs/08_dataset_info.ipynb 2
from .vision_core import *

from sklearn.utils.class_weight import compute_class_weight
from concurrent.futures import ThreadPoolExecutor
import pandas as pd
import numpy as np
import torch
import glob

# %% ../nbs/08_dataset_info.ipynb 4
class MedDataset:
    """A class to extract and present information about the dataset."""

    def __init__(self, path=None, postfix: str = '', img_list: list = None,
                 reorder: bool = False, dtype: (MedImage, MedMask) = MedImage,
                 max_workers: int = 1):
        """Constructs MedDataset object.

        Args:
            path (str, optional): Path to the image folder.
            postfix (str, optional): Specify the file type if there are different files in the folder.
            img_list (List[str], optional): Alternatively, pass in a list with image paths.
            reorder (bool, optional): Whether to reorder the data to be closest to canonical (RAS+) orientation.
            dtype (Union[MedImage, MedMask], optional): Load data as datatype. Default is MedImage.
            max_workers (int, optional): The number of worker threads. Default is 1.
        """
                     
        self.path = path
        self.postfix = postfix
        self.img_list = img_list
        self.reorder = reorder
        self.dtype = dtype
        self.max_workers = max_workers
        self.df = self._create_data_frame()

    def _create_data_frame(self):
        """Private method that returns a dataframe with information about the dataset."""

        if self.path:
            self.img_list = glob.glob(f'{self.path}/*{self.postfix}*')
            if not self.img_list: print('Could not find images. Check the image path')

        with ThreadPoolExecutor(max_workers=self.max_workers) as executor:
            data_info_dict = list(executor.map(self._get_data_info, self.img_list))

        df = pd.DataFrame(data_info_dict)
        
        if df.orientation.nunique() > 1:
            print('The volumes in this dataset have different orientations. '
                  'Recommended to pass in the argument reorder=True when creating a MedDataset object for this dataset')

        return df

    def summary(self):
        """Summary DataFrame of the dataset with example path for similar data."""
        
        columns = ['dim_0', 'dim_1', 'dim_2', 'voxel_0', 'voxel_1', 'voxel_2', 'orientation']
        
        return self.df.groupby(columns, as_index=False).agg(
            example_path=('path', 'min'), total=('path', 'size')
        ).sort_values('total', ascending=False)

    def suggestion(self):
        """Voxel value that appears most often in dim_0, dim_1 and dim_2, and whether the data should be reoriented."""
        
        resample = [self.df.voxel_0.mode()[0], self.df.voxel_1.mode()[0], self.df.voxel_2.mode()[0]]
        return resample, self.reorder

    def _get_data_info(self, fn: str):
        """Private method to collect information about an image file."""
        _, o, _ = med_img_reader(fn, dtype=self.dtype, reorder=self.reorder, only_tensor=False)

        info_dict = {'path': fn, 'dim_0': o.shape[1], 'dim_1': o.shape[2], 'dim_2': o.shape[3],
                     'voxel_0': round(o.spacing[0], 4), 'voxel_1': round(o.spacing[1], 4), 'voxel_2': round(o.spacing[2], 4),
                     'orientation': f'{"".join(o.orientation)}+'}

        if self.dtype is MedMask:
            mask_labels_dict = o.count_labels()
            mask_labels_dict = {f'voxel_count_{int(key)}': val for key, val in mask_labels_dict.items()}
            info_dict.update(mask_labels_dict)

        return info_dict

    def get_largest_img_size(self, resample: list = None) -> list:
        """Get the largest image size in the dataset."""
        
        dims = None

        if resample is not None:
            org_voxels = self.df[["voxel_0", "voxel_1", 'voxel_2']].values
            org_dims = self.df[["dim_0", "dim_1", 'dim_2']].values

            ratio = org_voxels/resample
            new_dims = (org_dims * ratio).T
            dims = [new_dims[0].max().round(), new_dims[1].max().round(), new_dims[2].max().round()]

        else:
            dims = [df.dim_0.max(), df.dim_1.max(), df.dim_2.max()]

        return dims

# %% ../nbs/08_dataset_info.ipynb 5
def get_class_weights(labels: (np.array, list), class_weight: str = 'balanced') -> torch.Tensor: 
    """Calculates and returns the class weights.

    Args:
        labels: An array or list of class labels for each instance in the dataset.
        class_weight: Defaults to 'balanced'.

    Returns:
        A tensor of class weights.
    """
    
    class_weights =  compute_class_weight(class_weight=class_weight, classes=np.unique(labels), y=labels)
    
    return torch.Tensor(class_weights)
